@@ -1,6 +1,6 @@
 /* GNU Chess 6 - main.cc - entry point
 
-   Copyright (c) 2001-2012 Free Software Foundation, Inc.
+   Copyright (c) 2001-2013 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs 
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -36,6 +36,7 @@ extern "C" {
 
 extern "C" {
 #include "frontend/common.h"
+#include "gettext.h"
 }
 
 unsigned char lzArray[65536];
@@ -76,8 +77,6 @@ HashType HashKey;
 HashType PawnHashKey;
 int Game50;
 unsigned long GenCnt;
-unsigned long NodeCnt;
-unsigned long QuiesCnt;
 char SANmv[SANSZ];
 char id[32];
 char solution[64];
@@ -89,7 +88,10 @@ int TCinc;
 float TCTime;
 int phase;
 short bookfirstlast;
-
+short graphicmodeoutput;
+short pgnloaded = 0;
+int pgncnt = 0;
+    
 char *progname;
 FILE *ofp;
 int myrating, opprating;
@@ -216,6 +218,15 @@ int main (int argc, char *argv[])
 {
   int i;
 
+  /* Set locale via LC_ALL.  */
+  setlocale (LC_ALL, "");
+
+#if ENABLE_NLS
+  /* Set the text message domain.  */
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+#endif
+
   /*
    * Parse command line arguments conforming with getopt_long syntax
    * Note: we have to support "xboard" and "post" as bare strings
@@ -224,10 +235,14 @@ int main (int argc, char *argv[])
  
   int c;
   int opt_help = 0, opt_version = 0, opt_post = 0, opt_xboard = 0, opt_memory = 0,
-      opt_easy = 0, opt_manual = 0, opt_quiet = 0, opt_uci = 0;
+      opt_easy = 0, opt_manual = 0, opt_quiet = 0, opt_uci = 0, opt_graphic = 0;
   char opt_addbook[MAXSTR+1] = "";
   char *endptr;
 
+  /*disable graphic output by default */
+  graphicmodeoutput = 0;
+  
+
   progname = argv[0]; /* Save in global for cmd_usage */
 
   while (1)
@@ -245,6 +260,7 @@ int main (int argc, char *argv[])
         {"manual", 0, 0, 'm'},
         {"uci", 0, 0, 'u'},
         {"addbook", 1, 0, 'a'},
+        {"graphic", 0, 0, 'g'},                
         {0, 0, 0, 0}
     };
  
@@ -252,7 +268,7 @@ int main (int argc, char *argv[])
 
     int option_index = 0;
  
-    c = getopt_long (argc, argv, "qehmpvxM:ua:",
+    c = getopt_long (argc, argv, "qehmpvxgM:ua:",
              long_options, &option_index);
  
     /* Detect the end of the options. */
@@ -284,9 +300,12 @@ int main (int argc, char *argv[])
      case 'e':
        opt_easy = 1;
        break;
+     case 'g':
+         opt_graphic = 1;
+       break;
      case 'm':
        opt_manual = 1;
-       break;
+       break;       
      case 'M':    
        if  ( optarg == NULL ){ /* we have error such as two -M */
          opt_help = 1;
@@ -359,7 +378,7 @@ int main (int argc, char *argv[])
   
   /* Startup output */
   if ( !( flags & XBOARD ) && ( !opt_quiet ) && ( !opt_uci) ) {
-    printf( "Copyright (C) 2012 Free Software Foundation, Inc.\n" );
+    printf( "Copyright (C) 2013 Free Software Foundation, Inc.\n" );
     printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
     printf( "This is free software: you are free to change and redistribute it.\n" );
     printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
@@ -429,6 +448,11 @@ int main (int argc, char *argv[])
 	SET (flags, POST);
       } 
     }
+
+  if ( opt_graphic == 1) {
+    graphicmodeoutput = 1;
+  }
+  
   }
 
   bookfirstlast = 3;
@@ -475,6 +499,8 @@ int main (int argc, char *argv[])
       /* Check if engine input ready for reading. If so, store it in a buffer. */
       ReadFromEngine();
     }
+    /* Avoid using 100% CPU */
+    usleep(100);
   }
   
   dbg_close();
